/*
 *   Copyright 2009 Joubin Houshyar
 * 
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *    
 *   http://www.apache.org/licenses/LICENSE-2.0
 *    
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.jredis.ri.alphazero.wip;


/**
 *
 * @author  joubin (alphazero@sensesay.net)
 * @date    Sep 6, 2009
 * 
 */

public class RoundRobinScheduler <T> implements Scheduler<T>{
	
	// ------------------------------------------------------------------------
	// Test
	// ------------------------------------------------------------------------
	private volatile int idx = 0;
	private final T[] resources;
	// ------------------------------------------------------------------------
	// Constructor
	// ------------------------------------------------------------------------
	/**
	 * New scheduler for scheduling items in resources array.
	 * @param resources
	 */
	public RoundRobinScheduler(T[] resources){ 
		this.resources = resources;
	}
	// ------------------------------------------------------------------------
	// Interface
	// ------------------------------------------------------------------------
	/**
	 * Resources are returned per round robin scheduling.  This method makes no
	 * guarantees whatsoever about the availability of the resources (which is
	 * resource <code>T</code> specific.
	 * 
	 * @return the next resource
	 * @see org.jredis.ri.alphazero.wip.Scheduler#getNext()
	 */
	public T getNext() {
		return resources[(++idx)%resources.length];
	}
	
	// ------------------------------------------------------------------------
	// Test
	// ------------------------------------------------------------------------
	public static void main (String[] args) {
		int n = 10;
		String[] connections = new String[n];
		for(int i=0; i<n; i++) connections[i]=String.valueOf(i);

	    Scheduler<String> roroSchedular = new RoundRobinScheduler<String>(connections);
	    int tn = 400;
	    for(int i=0; i<tn; i++)
	    	new Thread(getRunner(roroSchedular)).start();
    }
	private static Runnable getRunner (final Scheduler<?> scheduler) {
		return new Runnable() {
//			@Override 
			public void run() {
				Object conn = null;
				while(true){
					conn = scheduler.getNext();
					System.out.format("%s\n", conn.toString());			
				}
			}
		};
	}
}
